@@ -1,21 +1,26 @@
 import os
+import sys
 from importlib.metadata import metadata
 
 import requests.exceptions
 import typer
-from chrisclient2.chrisclient import ChrisClient, ChrisIncorrectLoginError, \
-    PipelineNotFoundError, run_pipeline_generator
-from chrisclient2.models import Pipeline, PluginInstance
+from chris.models import Pipeline, InvalidFilesResourceUrlException
 from typing import Optional, List
 import logging
 from pathlib import Path
 
 from caw.movedata import upload as cube_upload, download as cube_download
+from caw.login import LoginManager
+from caw.globals import DEFAULT_ADDRESS, DEFAULT_USERNAME, DEFAULT_PASSWORD
+from caw.helpers import ClientPrecursor, run_pipeline
 
-# logging.basicConfig(level=logging.DEBUG)
+if 'CAW_DEBUG' in os.environ:
+    logging.basicConfig(level=logging.DEBUG)
 logger = logging.getLogger(__name__)
 
-client: Optional[ChrisClient] = None
+
+login_manager = LoginManager()
+precursor = ClientPrecursor(login_manager)
 
 app = typer.Typer(
     epilog='Examples and documentation at '
@@ -36,9 +41,9 @@ def show_version(value: bool):
 
 @app.callback()
 def main(
-        address: str = typer.Option('http://localhost:8000/api/v1/', '--address', '-a', envvar='CHRIS_URL'),
-        username: str = typer.Option('chris', '--username', '-u', envvar='CHRIS_USERNAME'),
-        password: str = typer.Option('chris1234', '--password', '-p', envvar='CHRIS_PASSWORD'),
+        address: str = typer.Option(DEFAULT_ADDRESS, '--address', '-a', envvar='CHRIS_URL'),
+        username: str = typer.Option(DEFAULT_USERNAME, '--username', '-u', envvar='CHRIS_USERNAME'),
+        password: str = typer.Option(DEFAULT_PASSWORD, '--password', '-p', envvar='CHRIS_PASSWORD'),
         version: Optional[bool] = typer.Option(None, '--version', '-V',
                                                callback=show_version, is_eager=True,
                                                help='Print version.')
@@ -46,20 +51,37 @@ def main(
     """
     A command line ChRIS client for pipeline execution and data management.
     """
-    if 'CHRIS_TESTING' not in os.environ and password == 'chris1234':
-        typer.secho('Using defaults (set CHRIS_TESTING=y to supress this message): '
-                    f'{address}  {username}:{password}', dim=True, err=True)
-    global client
-    try:
-        client = ChrisClient(address, username, password)
-    except ChrisIncorrectLoginError as e:
-        typer.secho(e.args[0])
-        raise typer.Abort()
-    except Exception:
-        typer.secho('Connection error\n'
-                    f'address:  {address}\n'
-                    f'username: {username}', fg=typer.colors.RED, err=True)
-        raise typer.Abort()
+    global precursor
+    precursor.address = address
+    precursor.username = username
+    precursor.password = password
+
+
+@app.command()
+def login(read_pass: bool = typer.Option(False, '--password-stdin', help='Take the password from stdin')):
+    """
+    Login to ChRIS.
+    """
+    if precursor.username == DEFAULT_USERNAME:
+        precursor.username = typer.prompt('username')
+    if read_pass:
+        precursor.password = ('\n'.join(sys.stdin)).rstrip('\n')
+    elif precursor.password == DEFAULT_PASSWORD:
+        precursor.password = typer.prompt('password', hide_input=True)
+
+    client = precursor()
+    login_manager.login(client.addr, client.token)
+
+
+@app.command()
+def logout():
+    """
+    Remove your login credentials.
+    """
+    if precursor.address == DEFAULT_ADDRESS:
+        login_manager.logout()
+    else:
+        login_manager.logout(precursor.address)
 
 
 @app.command()
@@ -67,23 +89,9 @@ def search(name: str = typer.Argument('', help='name of pipeline to search for')
     """
     Search for pipelines that are saved in ChRIS.
     """
-    for pipeline in client.search_pipelines(name):
-        typer.echo(f'{pipeline.url:<60}{typer.style(pipeline.name, bold=True)}')
-
-
-def get_pipeline(name: str) -> Pipeline:
-    try:
-        return client.get_pipeline(name)
-    except PipelineNotFoundError:
-        typer.secho(f'Pipeline not found: "{pipeline}"', fg=typer.colors.RED, err=True)
-        raise typer.Abort()
-
-
-def run_pipeline(chris_pipeline: Pipeline, plugin_instance: PluginInstance):
-    with typer.progressbar(run_pipeline_generator(pipeline=chris_pipeline, plugin_instance=plugin_instance),
-                           length=len(chris_pipeline.pipings), label='Scheduling pipeline') as proto_pipeline:
-        for _ in proto_pipeline:
-            pass
+    client = precursor()
+    for search_result in client.search_pipelines(name):
+        typer.echo(f'{search_result.url:<60}{typer.style(search_result.name, bold=True)}')
 
 
 @app.command()
@@ -92,8 +100,9 @@ def pipeline(name: str = typer.Argument(..., help='Name of pipeline to run.'),
     """
     Run a pipeline on an existing feed.
     """
+    client = precursor()
     plugin_instance = client.get_plugin_instance(target)
-    chris_pipeline = get_pipeline(name)
+    chris_pipeline = client.get_pipeline(name)
     run_pipeline(chris_pipeline=chris_pipeline, plugin_instance=plugin_instance)
 
 
@@ -103,20 +112,22 @@ def upload(
         create_feed: bool = typer.Option(True, help='Run pl-dircopy on the newly uploaded files.'),
         name: str = typer.Option('', '--name', '-n', help='Name of the feed.'),
         description: str = typer.Option('', '--description', '-d', help='Description of the feed.'),
-        pipeline: str = typer.Option('', '--pipeline', '-p', help='Name of pipeline to run on the data.'),
+        pipeline_name: str = typer.Option('', '--pipeline', '-p', help='Name of pipeline to run on the data.'),
         files: List[Path] = typer.Argument(..., help='Files to upload. '
                                                      'Folder upload is supported, but directories are destructured.')
 ):
     """
-    Upload local files from host into ChRIS storage and then run pl-dircopy.
+    Upload local files and run pl-dircopy.
     """
+    client = precursor()
     chris_pipeline: Optional[Pipeline] = None
-    if pipeline:
-        chris_pipeline = get_pipeline(pipeline)
+    if pipeline_name:
+        chris_pipeline = client.get_pipeline(pipeline_name)
 
     try:
         swift_path = cube_upload(client=client, files=files, upload_threads=threads)
-    except requests.exceptions.RequestException:
+    except requests.exceptions.RequestException as e:
+        logger.debug('RequestException: %s\n%s', str(e), e.response.text)
         typer.secho('Upload unsuccessful', fg=typer.colors.RED, err=True)
         raise typer.Abort()
 
@@ -129,10 +140,9 @@ def upload(
     if description:
         dircopy_instance.get_feed().set_description(description)
 
-    if not chris_pipeline:
-        typer.echo(dircopy_instance.url)
-        raise typer.Exit()
-    run_pipeline(chris_pipeline=chris_pipeline, plugin_instance=dircopy_instance)
+    if chris_pipeline:
+        run_pipeline(chris_pipeline=chris_pipeline, plugin_instance=dircopy_instance)
+    typer.echo(dircopy_instance.feed)
 
 
 @app.command()
@@ -144,7 +154,12 @@ def download(
     """
     Download everything from a ChRIS url.
     """
-    cube_download(client=client, url=url, destination=destination, threads=threads)
+    client = precursor()
+    try:
+        cube_download(client=client, url=url, destination=destination, threads=threads)
+    except InvalidFilesResourceUrlException as e:
+        typer.secho(str(e), fg=typer.colors.RED, err=True)
+        raise typer.Abort()
 
 
 if __name__ == '__main__':
